=== modified file 'add_window.c'
--- add_window.c	2016-09-13 21:56:56 +0000
+++ add_window.c	2016-09-18 01:41:09 +0000
@@ -77,7 +77,6 @@
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
 
-#include "ctwm_atoms.h"
 #include "add_window.h"
 #include "colormaps.h"
 #include "windowbox.h"
@@ -97,7 +96,9 @@
 #include "functions.h"  // Only for RootFunction
 #include "decorations.h"
 #include "captive.h"
+#include "win_ops.h"
 #include "win_regions.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 
 #include "gram.tab.h"
@@ -117,46 +118,6 @@
 char NoName[] = "Untitled"; /* name if no name is specified */
 bool resizeWhenAdd;
 
-/************************************************************************
- *
- *  Procedure:
- *      GetGravityOffsets - map gravity to (x,y) offset signs for adding
- *              to x and y when window is mapped to get proper placement.
- *
- ************************************************************************
- */
-
-void GetGravityOffsets(TwmWindow *tmp,  /* window from which to get gravity */
-                       int *xp, int *yp)       /* return values */
-{
-	static struct _gravity_offset {
-		int x, y;
-	} gravity_offsets[11] = {
-		{  0,  0 },                     /* ForgetGravity */
-		{ -1, -1 },                     /* NorthWestGravity */
-		{  0, -1 },                     /* NorthGravity */
-		{  1, -1 },                     /* NorthEastGravity */
-		{ -1,  0 },                     /* WestGravity */
-		{  0,  0 },                     /* CenterGravity */
-		{  1,  0 },                     /* EastGravity */
-		{ -1,  1 },                     /* SouthWestGravity */
-		{  0,  1 },                     /* SouthGravity */
-		{  1,  1 },                     /* SouthEastGravity */
-		{  0,  0 },                     /* StaticGravity */
-	};
-	int g = ((tmp->hints.flags & PWinGravity)
-	         ? tmp->hints.win_gravity : NorthWestGravity);
-
-	if(g < ForgetGravity || g > StaticGravity) {
-		*xp = *yp = 0;
-	}
-	else {
-		*xp = gravity_offsets[g].x;
-		*yp = gravity_offsets[g].y;
-	}
-}
-
-
 
 
 /***********************************************************************
@@ -1500,39 +1461,21 @@
 	return (tmp_win);
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *      GetTwmWindow - finds the TwmWindow structure associated with
- *              a Window (if any), or NULL.
- *
- *  Returned Value:
- *      NULL    - it is not a Window we know about
- *      otherwise- the TwmWindow *
- *
- *  Inputs:
- *      w       - the window to check
- *
- *  Note:
- *      This is a relatively cheap function since it does not involve
- *      communication with the server. Probably faster than walking
- *      the list of TwmWindows, since the lookup is by a hash table.
- *
- ***********************************************************************
+
+/*
+ * XXX GrabButtons() and GrabKeys() are in a slightly odd state.  They're
+ * almost strictly a piece of the window-adding process, which is why
+ * they're here.  They're not static because the icon manager setup in
+ * CreateIconManagers() calls them explicitly, because they're also
+ * explicitly skipped in AddWindow() for icon manager windows.  I'm not
+ * sure that's necessary; x-ref comment in CIM() about some ideas on the
+ * matter.
+ *
+ * This should be resolved.  Until it is, they're left exported so the
+ * iconmgr code can all them, and they're left here (rather than moved to
+ * win_utils) on the guess that it may well be resolvable and so they'd
+ * stay here and be staticized in the end.
  */
-TwmWindow *GetTwmWindow(Window w)
-{
-	TwmWindow *twmwin;
-	int stat;
-
-	stat = XFindContext(dpy, w, TwmContext, (XPointer *)&twmwin);
-	if(stat == XCNOENT) {
-		twmwin = NULL;
-	}
-
-	return twmwin;
-}
-
 
 /***********************************************************************
  *
@@ -1739,97 +1682,6 @@
 #undef ungrabkey
 
 
-
-void FetchWmProtocols(TwmWindow *tmp)
-{
-	unsigned long flags = 0L;
-	Atom *protocols = NULL;
-	int n;
-
-	if(XGetWMProtocols(dpy, tmp->w, &protocols, &n)) {
-		int i;
-		Atom *ap;
-
-		for(i = 0, ap = protocols; i < n; i++, ap++) {
-			if(*ap == XA_WM_TAKE_FOCUS) {
-				flags |= DoesWmTakeFocus;
-			}
-			if(*ap == XA_WM_SAVE_YOURSELF) {
-				flags |= DoesWmSaveYourself;
-			}
-			if(*ap == XA_WM_DELETE_WINDOW) {
-				flags |= DoesWmDeleteWindow;
-			}
-		}
-		if(protocols) {
-			XFree(protocols);
-		}
-	}
-	tmp->protocols = flags;
-}
-
-
-void GetWindowSizeHints(TwmWindow *tmp)
-{
-	long supplied = 0;
-	XSizeHints *hints = &tmp->hints;
-
-	if(!XGetWMNormalHints(dpy, tmp->w, hints, &supplied)) {
-		hints->flags = 0;
-	}
-
-	if(hints->flags & PResizeInc) {
-		if(hints->width_inc == 0) {
-			hints->width_inc = 1;
-		}
-		if(hints->height_inc == 0) {
-			hints->height_inc = 1;
-		}
-	}
-
-	if(!(supplied & PWinGravity) && (hints->flags & USPosition)) {
-		static int gravs[] = { SouthEastGravity, SouthWestGravity,
-		                       NorthEastGravity, NorthWestGravity
-		                     };
-		int right =  tmp->attr.x + tmp->attr.width + 2 * tmp->old_bw;
-		int bottom = tmp->attr.y + tmp->attr.height + 2 * tmp->old_bw;
-		hints->win_gravity =
-		        gravs[((Scr->rooth - bottom <
-		                tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 2) |
-		              ((Scr->rootw - right   <
-		                tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 1)];
-		hints->flags |= PWinGravity;
-	}
-
-	/* Check for min size < max size */
-	if((hints->flags & (PMinSize | PMaxSize)) == (PMinSize | PMaxSize)) {
-		if(hints->max_width < hints->min_width) {
-			if(hints->max_width > 0) {
-				hints->min_width = hints->max_width;
-			}
-			else if(hints->min_width > 0) {
-				hints->max_width = hints->min_width;
-			}
-			else {
-				hints->max_width = hints->min_width = 1;
-			}
-		}
-
-		if(hints->max_height < hints->min_height) {
-			if(hints->max_height > 0) {
-				hints->min_height = hints->max_height;
-			}
-			else if(hints->min_height > 0) {
-				hints->max_height = hints->min_height;
-			}
-			else {
-				hints->max_height = hints->min_height = 1;
-			}
-		}
-	}
-}
-
-
 /*
  * This is largely for Xinerama support with VirtualScreens.
  * In this case, windows may be on something other then the main screen
@@ -1894,7 +1746,8 @@
  * Figure out if a window is a transient, and stash what it's transient
  * for.
  *
- * Previously in events.c.  Maybe fodder for a win_utils file?
+ * Previously in events.c.  Strictly win_utils-ish, but only used in
+ * AddWindow(), so might as well leave it here and static for now.
  */
 static bool
 Transient(Window w, Window *propw)

=== modified file 'add_window.h'
--- add_window.h	2016-08-30 19:34:30 +0000
+++ add_window.h	2016-09-17 00:38:47 +0000
@@ -75,20 +75,16 @@
 	AWT_OCCUPY,
 } AWType;
 
-void GetGravityOffsets(TwmWindow *tmp, int *xp, int *yp);
 TwmWindow *AddWindow(Window w, AWType wtype, IconMgr *iconp,
                      VirtualScreen *vs);
-TwmWindow *GetTwmWindow(Window w);
 void GrabButtons(TwmWindow *tmp_win);
 void GrabKeys(TwmWindow *tmp_win);
-void GetWindowSizeHints(TwmWindow *tmp_win);
 
 extern int AddingX;
 extern int AddingY;
 extern unsigned int AddingW;
 extern unsigned int AddingH;
 
-void FetchWmProtocols(TwmWindow *tmp);
 
 #endif /* _CTWM_ADD_WINDOW_H */
 

=== modified file 'animate.c'
--- animate.c	2016-09-07 07:17:07 +0000
+++ animate.c	2016-09-18 02:56:44 +0000
@@ -17,6 +17,7 @@
 #include "screen.h"
 #include "menus.h"
 #include "util.h"
+#include "win_utils.h"
 
 #include "animate.h"
 

=== modified file 'clicktofocus.c'
--- clicktofocus.c	2016-06-09 14:59:58 +0000
+++ clicktofocus.c	2016-09-18 01:47:05 +0000
@@ -1,7 +1,7 @@
 #include "ctwm.h"
-#include "util.h"
 #include "screen.h"
 #include "clicktofocus.h"
+#include "win_ops.h"
 
 static TwmWindow *get_last_window(WorkSpace *current)
 {

=== modified file 'cmake_files/ctwm_cmake_vars.cmake'
--- cmake_files/ctwm_cmake_vars.cmake	2016-09-07 07:32:23 +0000
+++ cmake_files/ctwm_cmake_vars.cmake	2016-09-18 01:41:09 +0000
@@ -51,7 +51,9 @@
 	util.c
 	vscreen.c
 	win_iconify.c
+	win_ops.c
 	win_regions.c
+	win_utils.c
 	windowbox.c
 	workspace_config.c
 	workspace_manager.c

=== modified file 'ctwm.c'
--- ctwm.c	2016-09-13 21:56:56 +0000
+++ ctwm.c	2016-09-18 02:49:41 +0000
@@ -92,7 +92,6 @@
 #include "gc.h"
 #include "parse.h"
 #include "version.h"
-#include "menus.h"
 #include "colormaps.h"
 #include "decorations_init.h"
 #include "events.h"
@@ -107,7 +106,9 @@
 #include "cursor.h"
 #include "windowbox.h"
 #include "captive.h"
+#include "win_ops.h"
 #include "win_regions.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #ifdef SOUNDS
 #  include "sound.h"

=== modified file 'decorations.c'
--- decorations.c	2016-08-15 07:45:02 +0000
+++ decorations.c	2016-09-18 02:58:07 +0000
@@ -15,8 +15,8 @@
 #include "image.h"
 #include "iconmgr.h"
 #include "screen.h"
-#include "util.h"
 #include "drawing.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 
 #include "decorations.h"
@@ -1480,6 +1480,90 @@
 }
 
 
+/*
+ * Setup the mouse cursor for various locations on the border of a
+ * window.
+ *
+ * Formerly in util.c
+ */
+void
+SetBorderCursor(TwmWindow *tmp_win, int x, int y)
+{
+	Cursor cursor;
+	XSetWindowAttributes attr;
+	int h, fw, fh, wd;
+
+	if(!tmp_win) {
+		return;
+	}
+
+	/* Use the max of these, but since one is always 0 we can add them. */
+	wd = tmp_win->frame_bw + tmp_win->frame_bw3D;
+	h = Scr->TitleHeight + wd;
+	fw = tmp_win->frame_width;
+	fh = tmp_win->frame_height;
+
+#if defined DEBUG && DEBUG
+	fprintf(stderr, "wd=%d h=%d, fw=%d fh=%d x=%d y=%d\n",
+	        wd, h, fw, fh, x, y);
+#endif
+
+	/*
+	 * If not using 3D borders:
+	 *
+	 * The left border has negative x coordinates,
+	 * The top border (above the title) has negative y coordinates.
+	 * The title is TitleHeight high, the next wd pixels are border.
+	 * The bottom border has coordinates >= the frame height.
+	 * The right border has coordinates >= the frame width.
+	 *
+	 * If using 3D borders: all coordinates are >= 0, and all coordinates
+	 * are higher by the border width.
+	 *
+	 * Since we only get events when we're actually in the border, we simply
+	 * allow for both cases at the same time.
+	 */
+
+	if((x < -wd) || (y < -wd)) {
+		cursor = Scr->FrameCursor;
+	}
+	else if(x < h) {
+		if(y < h) {
+			cursor = TopLeftCursor;
+		}
+		else if(y >= fh - h) {
+			cursor = BottomLeftCursor;
+		}
+		else {
+			cursor = LeftCursor;
+		}
+	}
+	else if(x >= fw - h) {
+		if(y < h) {
+			cursor = TopRightCursor;
+		}
+		else if(y >= fh - h) {
+			cursor = BottomRightCursor;
+		}
+		else {
+			cursor = RightCursor;
+		}
+	}
+	else if(y < h) {    /* also include title bar in top border area */
+		cursor = TopCursor;
+	}
+	else if(y >= fh - h) {
+		cursor = BottomCursor;
+	}
+	else {
+		cursor = Scr->FrameCursor;
+	}
+	attr.cursor = cursor;
+	XChangeWindowAttributes(dpy, tmp_win->frame, CWCursor, &attr);
+	tmp_win->curcurs = cursor;
+}
+
+
 
 /*
  * End of code.  Random doc/notes follow.

=== modified file 'decorations.h'
--- decorations.h	2016-06-09 15:06:43 +0000
+++ decorations.h	2016-09-18 01:53:29 +0000
@@ -21,6 +21,7 @@
 void PaintTitleButton(TwmWindow *tmp_win, TBWindow  *tbw);
 
 void PaintBorders(TwmWindow *tmp_win, bool focus);
+void SetBorderCursor(TwmWindow *tmp_win, int x, int y);
 
 
 #endif /* _CTWM_DECORATIONS_H */

=== modified file 'event_core.c'
--- event_core.c	2016-09-13 21:56:56 +0000
+++ event_core.c	2016-09-17 00:40:36 +0000
@@ -78,7 +78,6 @@
 
 #include <X11/extensions/shape.h>
 
-#include "add_window.h"
 #include "animate.h"
 #include "captive.h"
 #include "colormaps.h"
@@ -92,6 +91,7 @@
 #include "screen.h"
 #include "util.h"
 #include "version.h"
+#include "win_utils.h"
 #ifdef SOUNDS
 #include "sound.h"
 #endif

=== modified file 'event_handlers.c'
--- event_handlers.c	2016-09-13 21:57:26 +0000
+++ event_handlers.c	2016-09-18 01:41:09 +0000
@@ -97,7 +97,9 @@
 #include "screen.h"
 #include "util.h"
 #include "win_iconify.h"
+#include "win_ops.h"
 #include "win_regions.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
 

=== modified file 'ewmh.c'
--- ewmh.c	2016-09-07 07:17:07 +0000
+++ ewmh.c	2016-09-18 01:47:05 +0000
@@ -62,14 +62,14 @@
 #include "events.h"
 #include "event_handlers.h"
 #include "icons.h"
-#include "add_window.h"
 #include "otp.h"
-#include "util.h"
 #include "parse.h"
 #include "resize.h"
 #include "image.h"
 #include "functions.h"
 #include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_utils.h"
 
 /* #define DEBUG_EWMH */

=== modified file 'functions.c'
--- functions.c	2016-09-07 07:17:07 +0000
+++ functions.c	2016-09-18 02:56:44 +0000
@@ -21,7 +21,6 @@
 #include "event_handlers.h"
 #include "iconmgr.h"
 #include "icons.h"
-#include "menus.h"
 #include "otp.h"
 #include "parse.h"
 #include "resize.h"
@@ -30,6 +29,8 @@
 #include "drawing.h"
 #include "version.h"
 #include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "windowbox.h"
 #include "captive.h"
 #include "workspace_manager.h"

=== modified file 'iconmgr.c'
--- iconmgr.c	2016-08-27 23:45:38 +0000
+++ iconmgr.c	2016-09-18 02:56:44 +0000
@@ -80,6 +80,7 @@
 #include "otp.h"
 #include "add_window.h"
 #include "gram.tab.h"
+#include "win_utils.h"
 
 
 /* Where we start drawing the name in the icon manager */
@@ -245,6 +246,18 @@
 		Scr->workSpaceMgr.workSpaceList->iconmgr = Scr->iconmgr;
 	}
 
+
+	/*
+	 * Grab buttons/keystrokes for icon managers appropriately.
+	 * Normally, this is done in AddWindow(), but it explicitly skips it
+	 * for icon managers.  It's not at all clear why GrabButtons() would
+	 * do so; I don't think it needs to.  GrabKeys() does do some looping
+	 * over the Scr->iconmgr list at the end though, so it's possible we
+	 * need to delay calling it until now when they list is all filled
+	 * up.  This needs further investigation; it may be that the special
+	 * case and this code can be removed.  X-ref comments in add_window.c
+	 * about it.
+	 */
 	for(q = Scr->iconmgr; q != NULL; q = q->nextv) {
 		for(p = q; p != NULL; p = p->next) {
 			GrabButtons(p->twm_win);

=== modified file 'icons.c'
--- icons.c	2016-08-26 20:50:12 +0000
+++ icons.c	2016-09-18 02:56:44 +0000
@@ -77,6 +77,7 @@
 #include "util.h"
 #include "animate.h"
 #include "image.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 
 static void splitIconRegionEntry(IconEntry *ie, RegGravity grav1,

=== modified file 'mask_screen.c'
--- mask_screen.c	2016-08-07 02:48:41 +0000
+++ mask_screen.c	2016-09-18 02:58:07 +0000
@@ -8,13 +8,13 @@
 
 #include <sys/select.h>
 
-#include "util.h"
 #include "decorations.h"
 #include "screen.h"
 #include "icons.h"
 #include "cursor.h"
 #include "image.h"
 #include "gram.tab.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 
 #include "mask_screen.h"

=== modified file 'menus.c'
--- menus.c	2016-09-13 21:56:56 +0000
+++ menus.c	2016-09-19 02:37:09 +0000
@@ -86,12 +86,13 @@
 #include "icons.h"
 #include "add_window.h"
 #include "colormaps.h"
-#include "decorations.h"
 #include "otp.h"
 #include "image.h"
 #include "functions.h"
 #include "screen.h"
 #include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
 #ifdef SOUNDS
@@ -117,23 +118,11 @@
 static Cursor LastCursor;
 static bool addingdefaults = false;
 
-/*
- * Directionals for TryToPush_be().  These differ from the specs for
- * jump/pack/fill in functions. because there's an indeterminate option.
- */
-typedef enum {
-	PD_ANY,
-	PD_BOTTOM,
-	PD_LEFT,
-	PD_RIGHT,
-	PD_TOP,
-} PushDirection;
 
 
 static void Paint3DEntry(MenuRoot *mr, MenuItem *mi, bool exposure);
 static void PaintNormalEntry(MenuRoot *mr, MenuItem *mi, bool exposure);
 static void DestroyMenu(MenuRoot *menu);
-static void TryToPush_be(TwmWindow *tmp_win, int x, int y, PushDirection dir);
 
 
 #define SHADOWWIDTH 5                   /* in pixels */
@@ -1642,87 +1631,6 @@
 /***********************************************************************
  *
  *  Procedure:
- *      resizeFromCenter -
- *
- ***********************************************************************
- */
-
-void resizeFromCenter(Window w, TwmWindow *tmp_win)
-{
-	int lastx, lasty, bw2;
-
-	bw2 = tmp_win->frame_bw * 2;
-	AddingW = tmp_win->attr.width + bw2 + 2 * tmp_win->frame_bw3D;
-	AddingH = tmp_win->attr.height + tmp_win->title_height + bw2 + 2 *
-	          tmp_win->frame_bw3D;
-
-	XGetGeometry(dpy, w, &JunkRoot, &origDragX, &origDragY,
-	             &DragWidth, &DragHeight,
-	             &JunkBW, &JunkDepth);
-
-	XWarpPointer(dpy, None, w,
-	             0, 0, 0, 0, DragWidth / 2, DragHeight / 2);
-	XQueryPointer(dpy, Scr->Root, &JunkRoot,
-	              &JunkChild, &JunkX, &JunkY,
-	              &AddingX, &AddingY, &JunkMask);
-
-	lastx = -10000;
-	lasty = -10000;
-
-	MenuStartResize(tmp_win, origDragX, origDragY, DragWidth, DragHeight);
-	while(1) {
-		XMaskEvent(dpy,
-		           ButtonPressMask | PointerMotionMask | ExposureMask, &Event);
-
-		if(Event.type == MotionNotify) {
-			/* discard any extra motion events before a release */
-			while(XCheckMaskEvent(dpy,
-			                      ButtonMotionMask | ButtonPressMask, &Event))
-				if(Event.type == ButtonPress) {
-					break;
-				}
-		}
-
-		if(Event.type == ButtonPress) {
-			MenuEndResize(tmp_win);
-			// Next line should be unneeded, done by MenuEndResize() ?
-			XMoveResizeWindow(dpy, w, AddingX, AddingY, AddingW, AddingH);
-			break;
-		}
-
-		if(Event.type != MotionNotify) {
-			DispatchEvent2();
-			if(Cancel) {
-				// ...
-				MenuEndResize(tmp_win);
-				return;
-			}
-			continue;
-		}
-
-		/*
-		 * XXX - if we are going to do a loop, we ought to consider
-		 * using multiple GXxor lines so that we don't need to
-		 * grab the server.
-		 */
-		XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
-		              &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
-
-		if(lastx != AddingX || lasty != AddingY) {
-			MenuDoResize(AddingX, AddingY, tmp_win);
-
-			lastx = AddingX;
-			lasty = AddingY;
-		}
-
-	}
-}
-
-
-
-/***********************************************************************
- *
- *  Procedure:
  *      ReGrab - regrab the pointer with the LastCursor;
  *
  ***********************************************************************
@@ -1742,154 +1650,6 @@
 }
 
 
-/***********************************************************************
- *
- *  Procedure:
- *      FocusOnRoot - put input focus on the root window
- *
- ***********************************************************************
- */
-
-void FocusOnRoot(void)
-{
-	SetFocus(NULL, EventTime);
-	InstallColormaps(0, &Scr->RootColormaps);
-	if(! Scr->ClickToFocus) {
-		Scr->FocusRoot = true;
-	}
-}
-
-
-void AutoSqueeze(TwmWindow *tmp_win)
-{
-	if(tmp_win->isiconmgr) {
-		return;
-	}
-	if(Scr->RaiseWhenAutoUnSqueeze && tmp_win->squeezed) {
-		OtpRaise(tmp_win, WinWin);
-	}
-	Squeeze(tmp_win);
-}
-
-void Squeeze(TwmWindow *tmp_win)
-{
-	long fx, fy, savex, savey;
-	int  neww, newh;
-	bool south;
-	int  grav = ((tmp_win->hints.flags & PWinGravity)
-	             ? tmp_win->hints.win_gravity : NorthWestGravity);
-	long eventMask;
-	if(tmp_win->squeezed) {
-		tmp_win->squeezed = False;
-#ifdef EWMH
-		EwmhSet_NET_WM_STATE(tmp_win, EWMH_STATE_SHADED);
-#endif /* EWMH */
-		if(!tmp_win->isicon) {
-			XMapWindow(dpy, tmp_win->w);
-		}
-		SetupWindow(tmp_win, tmp_win->actual_frame_x, tmp_win->actual_frame_y,
-		            tmp_win->actual_frame_width, tmp_win->actual_frame_height, -1);
-		ReMapTransients(tmp_win);
-		return;
-	}
-
-	newh = tmp_win->title_height + 2 * tmp_win->frame_bw3D;
-	if(newh < 3) {
-		XBell(dpy, 0);
-		return;
-	}
-	switch(grav) {
-		case SouthWestGravity :
-		case SouthGravity :
-		case SouthEastGravity :
-			south = true;
-			break;
-		default :
-			south = false;
-			break;
-	}
-	if(tmp_win->title_height && !tmp_win->AlwaysSqueezeToGravity) {
-		south = false;
-	}
-
-	tmp_win->squeezed = true;
-	tmp_win->actual_frame_width  = tmp_win->frame_width;
-	tmp_win->actual_frame_height = tmp_win->frame_height;
-	savex = fx = tmp_win->frame_x;
-	savey = fy = tmp_win->frame_y;
-	neww  = tmp_win->actual_frame_width;
-	if(south) {
-		fy += tmp_win->frame_height - newh;
-	}
-	if(tmp_win->squeeze_info) {
-		fx  += tmp_win->title_x - tmp_win->frame_bw3D;
-		neww = tmp_win->title_width + 2 * (tmp_win->frame_bw + tmp_win->frame_bw3D);
-	}
-
-	eventMask = mask_out_event(tmp_win->w, StructureNotifyMask);
-#ifdef EWMH
-	EwmhSet_NET_WM_STATE(tmp_win, EWMH_STATE_SHADED);
-#endif /* EWMH */
-	XUnmapWindow(dpy, tmp_win->w);
-	restore_mask(tmp_win->w, eventMask);
-
-	if(fx + neww >= Scr->rootw - Scr->BorderRight) {
-		fx = Scr->rootw - Scr->BorderRight - neww;
-	}
-	if(fy + newh >= Scr->rooth - Scr->BorderBottom) {
-		fy = Scr->rooth - Scr->BorderBottom - newh;
-	}
-	SetupWindow(tmp_win, fx, fy, neww, newh, -1);
-	tmp_win->actual_frame_x = savex;
-	tmp_win->actual_frame_y = savey;
-
-	/* Now make the group members disappear */
-	UnmapTransients(tmp_win, false, eventMask);
-}
-
-/*
- * Set WM_STATE; x-ref ICCCM section 4.1.3.1
- * https://tronche.com/gui/x/icccm/sec-4.html#s-4.1.3.1
- */
-void SetMapStateProp(TwmWindow *tmp_win, int state)
-{
-	unsigned long data[2];              /* "suggested" by ICCCM version 1 */
-
-	data[0] = (unsigned long) state;
-	data[1] = (unsigned long)(tmp_win->iconify_by_unmapping ? None :
-	                          (tmp_win->icon ? tmp_win->icon->w : None));
-
-	XChangeProperty(dpy, tmp_win->w, XA_WM_STATE, XA_WM_STATE, 32,
-	                PropModeReplace, (unsigned char *) data, 2);
-}
-
-
-bool
-GetWMState(Window w, int *statep, Window *iwp)
-{
-	Atom actual_type;
-	int actual_format;
-	unsigned long nitems, bytesafter;
-	unsigned long *datap = NULL;
-	bool retval = false;
-
-	if(XGetWindowProperty(dpy, w, XA_WM_STATE, 0L, 2L, False, XA_WM_STATE,
-	                      &actual_type, &actual_format, &nitems, &bytesafter,
-	                      (unsigned char **) &datap) != Success || !datap) {
-		return false;
-	}
-
-	if(nitems <= 2) {                   /* "suggested" by ICCCM version 1 */
-		*statep = (int) datap[0];
-		*iwp = (Window) datap[1];
-		retval = true;
-	}
-
-	XFree(datap);
-	return retval;
-}
-
-
 int WarpToScreen(int n, int inc)
 {
 	Window dumwin;
@@ -2247,235 +2007,6 @@
 	return;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *      DisplayPosition - display the position in the dimensions window
- *
- *  Inputs:
- *      tmp_win - the current twm window
- *      x, y    - position of the window
- *
- ***********************************************************************
- */
-
-void DisplayPosition(TwmWindow *tmp_win, int x, int y)
-{
-	char str [100];
-	char signx = '+';
-	char signy = '+';
-
-	if(x < 0) {
-		x = -x;
-		signx = '-';
-	}
-	if(y < 0) {
-		y = -y;
-		signy = '-';
-	}
-	sprintf(str, " %c%-4d %c%-4d ", signx, x, signy, y);
-	XRaiseWindow(dpy, Scr->SizeWindow);
-
-	Draw3DBorder(Scr->SizeWindow, 0, 0,
-	             Scr->SizeStringOffset + Scr->SizeStringWidth + SIZE_HINDENT,
-	             Scr->SizeFont.height + SIZE_VINDENT * 2,
-	             2, Scr->DefaultC, off, false, false);
-
-	FB(Scr->DefaultC.fore, Scr->DefaultC.back);
-	XmbDrawImageString(dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
-	                   Scr->NormalGC, Scr->SizeStringOffset,
-	                   Scr->SizeFont.ascent + SIZE_VINDENT , str, 13);
-}
-
-
-void TryToPack(TwmWindow *tmp_win, int *x, int *y)
-{
-	TwmWindow   *t;
-	int         newx, newy;
-	int         w, h;
-	int         winw = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
-	int         winh = tmp_win->frame_height + 2 * tmp_win->frame_bw;
-
-	newx = *x;
-	newy = *y;
-	for(t = Scr->FirstWindow; t != NULL; t = t->next) {
-		if(t == tmp_win) {
-			continue;
-		}
-		if(t->winbox != tmp_win->winbox) {
-			continue;
-		}
-		if(t->vs != tmp_win->vs) {
-			continue;
-		}
-		if(!t->mapped) {
-			continue;
-		}
-
-		w = t->frame_width  + 2 * t->frame_bw;
-		h = t->frame_height + 2 * t->frame_bw;
-		if(newx >= t->frame_x + w) {
-			continue;
-		}
-		if(newy >= t->frame_y + h) {
-			continue;
-		}
-		if(newx + winw <= t->frame_x) {
-			continue;
-		}
-		if(newy + winh <= t->frame_y) {
-			continue;
-		}
-
-		if(newx + Scr->MovePackResistance > t->frame_x + w) {  /* left */
-			newx = MAX(newx, t->frame_x + w);
-			continue;
-		}
-		if(newx + winw < t->frame_x + Scr->MovePackResistance) {  /* right */
-			newx = MIN(newx, t->frame_x - winw);
-			continue;
-		}
-		if(newy + Scr->MovePackResistance > t->frame_y + h) {  /* top */
-			newy = MAX(newy, t->frame_y + h);
-			continue;
-		}
-		if(newy + winh < t->frame_y + Scr->MovePackResistance) {  /* bottom */
-			newy = MIN(newy, t->frame_y - winh);
-			continue;
-		}
-	}
-	*x = newx;
-	*y = newy;
-}
-
-
-void
-TryToPush(TwmWindow *tmp_win, int x, int y)
-{
-	TryToPush_be(tmp_win, x, y, PD_ANY);
-}
-
-static void
-TryToPush_be(TwmWindow *tmp_win, int x, int y, PushDirection dir)
-{
-	TwmWindow   *t;
-	int         newx, newy, ndir;
-	bool        move;
-	int         w, h;
-	int         winw = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
-	int         winh = tmp_win->frame_height + 2 * tmp_win->frame_bw;
-
-	for(t = Scr->FirstWindow; t != NULL; t = t->next) {
-		if(t == tmp_win) {
-			continue;
-		}
-		if(t->winbox != tmp_win->winbox) {
-			continue;
-		}
-		if(t->vs != tmp_win->vs) {
-			continue;
-		}
-		if(!t->mapped) {
-			continue;
-		}
-
-		w = t->frame_width  + 2 * t->frame_bw;
-		h = t->frame_height + 2 * t->frame_bw;
-		if(x >= t->frame_x + w) {
-			continue;
-		}
-		if(y >= t->frame_y + h) {
-			continue;
-		}
-		if(x + winw <= t->frame_x) {
-			continue;
-		}
-		if(y + winh <= t->frame_y) {
-			continue;
-		}
-
-		move = false;
-		if((dir == PD_ANY || dir == PD_LEFT) &&
-		                (x + Scr->MovePackResistance > t->frame_x + w)) {
-			newx = x - w;
-			newy = t->frame_y;
-			ndir = PD_LEFT;
-			move = true;
-		}
-		else if((dir == PD_ANY || dir == PD_RIGHT) &&
-		                (x + winw < t->frame_x + Scr->MovePackResistance)) {
-			newx = x + winw;
-			newy = t->frame_y;
-			ndir = PD_RIGHT;
-			move = true;
-		}
-		else if((dir == PD_ANY || dir == PD_TOP) &&
-		                (y + Scr->MovePackResistance > t->frame_y + h)) {
-			newx = t->frame_x;
-			newy = y - h;
-			ndir = PD_TOP;
-			move = true;
-		}
-		else if((dir == PD_ANY || dir == PD_BOTTOM) &&
-		                (y + winh < t->frame_y + Scr->MovePackResistance)) {
-			newx = t->frame_x;
-			newy = y + winh;
-			ndir = PD_BOTTOM;
-			move = true;
-		}
-		if(move) {
-			TryToPush_be(t, newx, newy, ndir);
-			TryToPack(t, &newx, &newy);
-			ConstrainByBorders(tmp_win,
-			                   &newx, t->frame_width  + 2 * t->frame_bw,
-			                   &newy, t->frame_height + 2 * t->frame_bw);
-			SetupWindow(t, newx, newy, t->frame_width, t->frame_height, -1);
-		}
-	}
-}
-
-
-void TryToGrid(TwmWindow *tmp_win, int *x, int *y)
-{
-	int w    = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
-	int h    = tmp_win->frame_height + 2 * tmp_win->frame_bw;
-	int grav = ((tmp_win->hints.flags & PWinGravity)
-	            ? tmp_win->hints.win_gravity : NorthWestGravity);
-
-	switch(grav) {
-		case ForgetGravity :
-		case StaticGravity :
-		case NorthWestGravity :
-		case NorthGravity :
-		case WestGravity :
-		case CenterGravity :
-			*x = ((*x - Scr->BorderLeft) / Scr->XMoveGrid) * Scr->XMoveGrid
-			     + Scr->BorderLeft;
-			*y = ((*y - Scr->BorderTop) / Scr->YMoveGrid) * Scr->YMoveGrid
-			     + Scr->BorderTop;
-			break;
-		case NorthEastGravity :
-		case EastGravity :
-			*x = (((*x + w - Scr->BorderLeft) / Scr->XMoveGrid) *
-			      Scr->XMoveGrid) - w + Scr->BorderLeft;
-			*y = ((*y - Scr->BorderTop) / Scr->YMoveGrid) *
-			     Scr->YMoveGrid + Scr->BorderTop;
-			break;
-		case SouthWestGravity :
-		case SouthGravity :
-			*x = ((*x - Scr->BorderLeft) / Scr->XMoveGrid) * Scr->XMoveGrid
-			     + Scr->BorderLeft;
-			*y = (((*y + h - Scr->BorderTop) / Scr->YMoveGrid) * Scr->YMoveGrid)
-			     - h + Scr->BorderTop;
-			break;
-		case SouthEastGravity :
-			*x = (((*x + w - Scr->BorderLeft) / Scr->XMoveGrid) *
-			      Scr->XMoveGrid) - w + Scr->BorderLeft;
-			*y = (((*y + h - Scr->BorderTop) / Scr->YMoveGrid) *
-			      Scr->YMoveGrid) - h + Scr->BorderTop;
-			break;
-	}
-}
 
 void WarpCursorToDefaultEntry(MenuRoot *menu)
 {

=== modified file 'menus.h'
--- menus.h	2016-09-03 04:29:20 +0000
+++ menus.h	2016-09-19 02:37:09 +0000
@@ -197,7 +197,6 @@
 void AddDefaultFuncButtons(void);
 void PopDownMenu(void);
 void HideMenu(MenuRoot *menu);
-void resizeFromCenter(Window w, TwmWindow *tmp_win);
 void ReGrab(void);
 void SetLastCursor(Cursor newcur);
 void PaintEntry(MenuRoot *mr, MenuItem *mi, bool exposure);
@@ -207,8 +206,6 @@
 void MakeMenus(void);
 void MakeMenu(MenuRoot *mr);
 void MoveMenu(XEvent *eventp);
-void SetMapStateProp(TwmWindow *tmp_win, int state);
-bool GetWMState(Window w, int *statep, Window *iwp);
 void send_clientmessage(Window w, Atom a, Time timestamp);
 void SendEndAnimationMessage(Window w, Time timestamp);
 void SendTakeFocusMessage(TwmWindow *tmp, Time timestamp);
@@ -218,15 +215,8 @@
 void RaiseLowerFrame(Window frame, int ontop);
 void MapRaised(TwmWindow *tmp_win);
 void RaiseFrame(Window frame);
-void FocusOnRoot(void);
-void TryToPack(TwmWindow *tmp_win, int *x, int *y);
-void TryToPush(TwmWindow *tmp_win, int x, int y);
-void TryToGrid(TwmWindow *tmp_win, int *x, int *y);
 void WarpCursorToDefaultEntry(MenuRoot *menu);
 void WarpToWindow(TwmWindow *t, bool must_raise);
-void DisplayPosition(TwmWindow *tmp_win, int x, int y);
-void AutoSqueeze(TwmWindow *tmp_win);
-void Squeeze(TwmWindow *tmp_win);
 
 /* To move soonish? */
 void WarpAlongRing(XButtonEvent *ev, bool forward);

=== modified file 'occupation.c'
--- occupation.c	2016-09-03 04:29:20 +0000
+++ occupation.c	2016-09-17 00:38:47 +0000
@@ -28,6 +28,7 @@
 #include "util.h"
 #include "win_iconify.h"
 #include "win_regions.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
 

=== modified file 'resize.c'
--- resize.c	2016-09-13 21:56:56 +0000
+++ resize.c	2016-09-20 03:03:41 +0000
@@ -73,6 +73,7 @@
 #include <stdlib.h>
 
 #include "parse.h"
+#include "events.h"
 #include "util.h"
 #include "otp.h"
 #include "resize.h"
@@ -81,6 +82,8 @@
 #include "decorations.h"
 #include "screen.h"
 #include "drawing.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "iconmgr.h"
 
@@ -1277,3 +1280,83 @@
 		}
 	}
 }
+
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *      resizeFromCenter -
+ *
+ ***********************************************************************
+ */
+void
+resizeFromCenter(Window w, TwmWindow *tmp_win)
+{
+	int lastx, lasty, bw2;
+
+	bw2 = tmp_win->frame_bw * 2;
+	AddingW = tmp_win->attr.width + bw2 + 2 * tmp_win->frame_bw3D;
+	AddingH = tmp_win->attr.height + tmp_win->title_height + bw2 + 2 *
+	          tmp_win->frame_bw3D;
+
+	XGetGeometry(dpy, w, &JunkRoot, &origDragX, &origDragY,
+	             &DragWidth, &DragHeight,
+	             &JunkBW, &JunkDepth);
+
+	XWarpPointer(dpy, None, w,
+	             0, 0, 0, 0, DragWidth / 2, DragHeight / 2);
+	XQueryPointer(dpy, Scr->Root, &JunkRoot,
+	              &JunkChild, &JunkX, &JunkY,
+	              &AddingX, &AddingY, &JunkMask);
+
+	lastx = -10000;
+	lasty = -10000;
+
+	MenuStartResize(tmp_win, origDragX, origDragY, DragWidth, DragHeight);
+	while(1) {
+		XMaskEvent(dpy,
+		           ButtonPressMask | PointerMotionMask | ExposureMask, &Event);
+
+		if(Event.type == MotionNotify) {
+			/* discard any extra motion events before a release */
+			while(XCheckMaskEvent(dpy,
+			                      ButtonMotionMask | ButtonPressMask, &Event))
+				if(Event.type == ButtonPress) {
+					break;
+				}
+		}
+
+		if(Event.type == ButtonPress) {
+			MenuEndResize(tmp_win);
+			// Next line should be unneeded, done by MenuEndResize() ?
+			XMoveResizeWindow(dpy, w, AddingX, AddingY, AddingW, AddingH);
+			break;
+		}
+
+		if(Event.type != MotionNotify) {
+			DispatchEvent2();
+			if(Cancel) {
+				// ...
+				MenuEndResize(tmp_win);
+				return;
+			}
+			continue;
+		}
+
+		/*
+		 * XXX - if we are going to do a loop, we ought to consider
+		 * using multiple GXxor lines so that we don't need to
+		 * grab the server.
+		 */
+		XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
+		              &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
+
+		if(lastx != AddingX || lasty != AddingY) {
+			MenuDoResize(AddingX, AddingY, tmp_win);
+
+			lastx = AddingX;
+			lasty = AddingY;
+		}
+
+	}
+}

=== modified file 'resize.h'
--- resize.h	2016-06-09 15:06:43 +0000
+++ resize.h	2016-09-19 01:39:52 +0000
@@ -84,4 +84,6 @@
 
 void ChangeSize(char *in_string, TwmWindow *tmp_win);
 
+void resizeFromCenter(Window w, TwmWindow *tmp_win);
+
 #endif /* _CTWM_RESIZE_H */

=== modified file 'util.c'
--- util.c	2016-09-11 15:41:36 +0000
+++ util.c	2016-09-20 03:14:43 +0000
@@ -109,139 +109,6 @@
 FILE *tracefile = NULL;
 
 
-
-/***********************************************************************
- *
- *  Procedure:
- *      MoveOutline - move a window outline
- *
- *  Inputs:
- *      root        - the window we are outlining
- *      x           - upper left x coordinate
- *      y           - upper left y coordinate
- *      width       - the width of the rectangle
- *      height      - the height of the rectangle
- *      bw          - the border width of the frame
- *      th          - title height
- *
- ***********************************************************************
- */
-
-/* ARGSUSED */
-void MoveOutline(Window root,
-                 int x, int y, int width, int height, int bw, int th)
-{
-	static int  lastx = 0;
-	static int  lasty = 0;
-	static int  lastWidth = 0;
-	static int  lastHeight = 0;
-	static int  lastBW = 0;
-	static int  lastTH = 0;
-	int         xl, xr, yt, yb, xinnerl, xinnerr, yinnert, yinnerb;
-	int         xthird, ythird;
-	XSegment    outline[18];
-	XSegment   *r;
-
-	if(x == lastx && y == lasty && width == lastWidth && height == lastHeight
-	                && lastBW == bw && th == lastTH) {
-		return;
-	}
-
-	r = outline;
-
-#define DRAWIT() \
-    if (lastWidth || lastHeight)                        \
-    {                                                   \
-        xl = lastx;                                     \
-        xr = lastx + lastWidth - 1;                     \
-        yt = lasty;                                     \
-        yb = lasty + lastHeight - 1;                    \
-        xinnerl = xl + lastBW;                          \
-        xinnerr = xr - lastBW;                          \
-        yinnert = yt + lastTH + lastBW;                 \
-        yinnerb = yb - lastBW;                          \
-        xthird = (xinnerr - xinnerl) / 3;               \
-        ythird = (yinnerb - yinnert) / 3;               \
-                                                        \
-        r->x1 = xl;                                     \
-        r->y1 = yt;                                     \
-        r->x2 = xr;                                     \
-        r->y2 = yt;                                     \
-        r++;                                            \
-                                                        \
-        r->x1 = xl;                                     \
-        r->y1 = yb;                                     \
-        r->x2 = xr;                                     \
-        r->y2 = yb;                                     \
-        r++;                                            \
-                                                        \
-        r->x1 = xl;                                     \
-        r->y1 = yt;                                     \
-        r->x2 = xl;                                     \
-        r->y2 = yb;                                     \
-        r++;                                            \
-                                                        \
-        r->x1 = xr;                                     \
-        r->y1 = yt;                                     \
-        r->x2 = xr;                                     \
-        r->y2 = yb;                                     \
-        r++;                                            \
-                                                        \
-        r->x1 = xinnerl + xthird;                       \
-        r->y1 = yinnert;                                \
-        r->x2 = r->x1;                                  \
-        r->y2 = yinnerb;                                \
-        r++;                                            \
-                                                        \
-        r->x1 = xinnerl + (2 * xthird);                 \
-        r->y1 = yinnert;                                \
-        r->x2 = r->x1;                                  \
-        r->y2 = yinnerb;                                \
-        r++;                                            \
-                                                        \
-        r->x1 = xinnerl;                                \
-        r->y1 = yinnert + ythird;                       \
-        r->x2 = xinnerr;                                \
-        r->y2 = r->y1;                                  \
-        r++;                                            \
-                                                        \
-        r->x1 = xinnerl;                                \
-        r->y1 = yinnert + (2 * ythird);                 \
-        r->x2 = xinnerr;                                \
-        r->y2 = r->y1;                                  \
-        r++;                                            \
-                                                        \
-        if (lastTH != 0) {                              \
-            r->x1 = xl;                                 \
-            r->y1 = yt + lastTH;                        \
-            r->x2 = xr;                                 \
-            r->y2 = r->y1;                              \
-            r++;                                        \
-        }                                               \
-    }
-
-	/* undraw the old one, if any */
-	DRAWIT();
-
-	lastx = x;
-	lasty = y;
-	lastWidth = width;
-	lastHeight = height;
-	lastBW = bw;
-	lastTH = th;
-
-	/* draw the new one, if any */
-	DRAWIT();
-
-#undef DRAWIT
-
-
-	if(r != outline) {
-		XDrawSegments(dpy, root, Scr->DrawGC, outline, r - outline);
-	}
-}
-
-
 /*
  * Rewrite this, possibly in terms of replace_substr().  Alternately, the
  * places it's being used might be better served by being preprocessed
@@ -511,97 +378,6 @@
 }
 
 
-void
-SetFocusVisualAttributes(TwmWindow *tmp_win, bool focus)
-{
-	if(! tmp_win) {
-		return;
-	}
-
-	if(focus == tmp_win->hasfocusvisible) {
-		return;
-	}
-	if(tmp_win->highlight) {
-		if(Scr->use3Dborders) {
-			PaintBorders(tmp_win, focus);
-		}
-		else {
-			if(focus) {
-				XSetWindowBorder(dpy, tmp_win->frame, tmp_win->borderC.back);
-				if(tmp_win->title_w) {
-					XSetWindowBorder(dpy, tmp_win->title_w, tmp_win->borderC.back);
-				}
-			}
-			else {
-				/*
-				 * XXX It seems possible this could be replaced by a
-				 * single global 'gray' pixmap; I don't think it actually
-				 * varies per window, and I don't see any obvious reason
-				 * it can't be reused, so we may be able to save an
-				 * allocation for each window by doing so...
-				 */
-				XSetWindowBorderPixmap(dpy, tmp_win->frame, tmp_win->gray);
-				if(tmp_win->title_w) {
-					XSetWindowBorderPixmap(dpy, tmp_win->title_w, tmp_win->gray);
-				}
-			}
-		}
-	}
-
-	if(focus) {
-		bool hil = false;
-
-		if(tmp_win->lolite_wl) {
-			XUnmapWindow(dpy, tmp_win->lolite_wl);
-		}
-		if(tmp_win->lolite_wr) {
-			XUnmapWindow(dpy, tmp_win->lolite_wr);
-		}
-		if(tmp_win->hilite_wl) {
-			XMapWindow(dpy, tmp_win->hilite_wl);
-			hil = true;
-		}
-		if(tmp_win->hilite_wr) {
-			XMapWindow(dpy, tmp_win->hilite_wr);
-			hil = true;
-		}
-		if(hil && tmp_win->HiliteImage && tmp_win->HiliteImage->next) {
-			MaybeAnimate = true;
-		}
-		if(tmp_win->iconmanagerlist) {
-			ActiveIconManager(tmp_win->iconmanagerlist);
-		}
-	}
-	else {
-		if(tmp_win->hilite_wl) {
-			XUnmapWindow(dpy, tmp_win->hilite_wl);
-		}
-		if(tmp_win->hilite_wr) {
-			XUnmapWindow(dpy, tmp_win->hilite_wr);
-		}
-		if(tmp_win->lolite_wl) {
-			XMapWindow(dpy, tmp_win->lolite_wl);
-		}
-		if(tmp_win->lolite_wr) {
-			XMapWindow(dpy, tmp_win->lolite_wr);
-		}
-		if(tmp_win->iconmanagerlist) {
-			NotActiveIconManager(tmp_win->iconmanagerlist);
-		}
-	}
-	if(Scr->use3Dtitles && Scr->SunkFocusWindowTitle && tmp_win->title_height) {
-		ButtonState bs;
-
-		bs = focus ? on : off;
-		Draw3DBorder(tmp_win->title_w, Scr->TBInfo.titlex, 0,
-		             tmp_win->title_width - Scr->TBInfo.titlex -
-		             Scr->TBInfo.rightoff - Scr->TitlePadding,
-		             Scr->TitleHeight, Scr->TitleShadowDepth,
-		             tmp_win->title, bs, false, false);
-	}
-	tmp_win->hasfocusvisible = focus;
-}
-
 #if 0
 static void move_to_head(TwmWindow *t)
 {
@@ -668,45 +444,6 @@
 }
 #endif
 
-/*
- * SetFocus - separate routine to set focus to make things more understandable
- * and easier to debug
- */
-void SetFocus(TwmWindow *tmp_win, Time tim)
-{
-	Window w = (tmp_win ? tmp_win->w : PointerRoot);
-	bool f_iconmgr = false;
-
-	if(Scr->Focus && (Scr->Focus->isiconmgr)) {
-		f_iconmgr = true;
-	}
-	if(Scr->SloppyFocus && (w == PointerRoot) && (!f_iconmgr)) {
-		return;
-	}
-
-	XSetInputFocus(dpy, w, RevertToPointerRoot, tim);
-#ifdef EWMH
-	EwmhSet_NET_ACTIVE_WINDOW(w);
-#endif
-	if(Scr->Focus == tmp_win) {
-		return;
-	}
-
-	if(Scr->Focus) {
-		if(Scr->Focus->AutoSqueeze && !Scr->Focus->squeezed) {
-			AutoSqueeze(Scr->Focus);
-		}
-		SetFocusVisualAttributes(Scr->Focus, false);
-	}
-	if(tmp_win)    {
-		if(tmp_win->AutoSqueeze && tmp_win->squeezed) {
-			AutoSqueeze(tmp_win);
-		}
-		SetFocusVisualAttributes(tmp_win, true);
-	}
-	Scr->Focus = tmp_win;
-}
-
 
 void AdoptWindow(void)
 {
@@ -904,230 +641,6 @@
 }
 
 
-void SetBorderCursor(TwmWindow *tmp_win, int x, int y)
-{
-	Cursor cursor;
-	XSetWindowAttributes attr;
-	int h, fw, fh, wd;
-
-	if(!tmp_win) {
-		return;
-	}
-
-	/* Use the max of these, but since one is always 0 we can add them. */
-	wd = tmp_win->frame_bw + tmp_win->frame_bw3D;
-	h = Scr->TitleHeight + wd;
-	fw = tmp_win->frame_width;
-	fh = tmp_win->frame_height;
-
-#if defined DEBUG && DEBUG
-	fprintf(stderr, "wd=%d h=%d, fw=%d fh=%d x=%d y=%d\n",
-	        wd, h, fw, fh, x, y);
-#endif
-
-	/*
-	 * If not using 3D borders:
-	 *
-	 * The left border has negative x coordinates,
-	 * The top border (above the title) has negative y coordinates.
-	 * The title is TitleHeight high, the next wd pixels are border.
-	 * The bottom border has coordinates >= the frame height.
-	 * The right border has coordinates >= the frame width.
-	 *
-	 * If using 3D borders: all coordinates are >= 0, and all coordinates
-	 * are higher by the border width.
-	 *
-	 * Since we only get events when we're actually in the border, we simply
-	 * allow for both cases at the same time.
-	 */
-
-	if((x < -wd) || (y < -wd)) {
-		cursor = Scr->FrameCursor;
-	}
-	else if(x < h) {
-		if(y < h) {
-			cursor = TopLeftCursor;
-		}
-		else if(y >= fh - h) {
-			cursor = BottomLeftCursor;
-		}
-		else {
-			cursor = LeftCursor;
-		}
-	}
-	else if(x >= fw - h) {
-		if(y < h) {
-			cursor = TopRightCursor;
-		}
-		else if(y >= fh - h) {
-			cursor = BottomRightCursor;
-		}
-		else {
-			cursor = RightCursor;
-		}
-	}
-	else if(y < h) {    /* also include title bar in top border area */
-		cursor = TopCursor;
-	}
-	else if(y >= fh - h) {
-		cursor = BottomCursor;
-	}
-	else {
-		cursor = Scr->FrameCursor;
-	}
-	attr.cursor = cursor;
-	XChangeWindowAttributes(dpy, tmp_win->frame, CWCursor, &attr);
-	tmp_win->curcurs = cursor;
-}
-
-
-/***********************************************************************
- *
- *  Procedure:
- *      GetWMPropertyString - Get Window Manager text property and
- *                              convert it to a string.
- *
- *  Returned Value:
- *      (char *) - pointer to the malloc'd string or NULL
- *
- *  Inputs:
- *      w       - the id of the window whose property is to be retrieved
- *      prop    - property atom (typically WM_NAME or WM_ICON_NAME)
- *
- ***********************************************************************
- */
-
-unsigned char *GetWMPropertyString(Window w, Atom prop)
-{
-	XTextProperty       text_prop;
-	char                **text_list;
-	int                 text_list_count;
-	unsigned char       *stringptr;
-	int                 status, len = -1;
-
-	(void)XGetTextProperty(dpy, w, &text_prop, prop);
-	if(text_prop.value != NULL) {
-		if(text_prop.encoding == XA_STRING
-		                || text_prop.encoding == XA_COMPOUND_TEXT) {
-			/* property is encoded as compound text - convert to locale string */
-			status = XmbTextPropertyToTextList(dpy, &text_prop,
-			                                   &text_list, &text_list_count);
-			if(text_list_count == 0) {
-				stringptr = NULL;
-			}
-			else if(text_list == NULL) {
-				stringptr = NULL;
-			}
-			else if(text_list [0] == NULL) {
-				stringptr = NULL;
-			}
-			else if(status < 0 || text_list_count < 0) {
-				switch(status) {
-					case XConverterNotFound:
-						fprintf(stderr,
-						        "%s: Converter not found; unable to convert property %s of window ID %lx.\n",
-						        ProgramName, XGetAtomName(dpy, prop), w);
-						break;
-					case XNoMemory:
-						fprintf(stderr,
-						        "%s: Insufficient memory; unable to convert property %s of window ID %lx.\n",
-						        ProgramName, XGetAtomName(dpy, prop), w);
-						break;
-					case XLocaleNotSupported:
-						fprintf(stderr,
-						        "%s: Locale not supported; unable to convert property %s of window ID %lx.\n",
-						        ProgramName, XGetAtomName(dpy, prop), w);
-						break;
-				}
-				stringptr = NULL;
-				/*
-				   don't call XFreeStringList - text_list appears to have
-				   invalid address if status is bad
-				   XFreeStringList(text_list);
-				*/
-			}
-			else {
-				len = strlen(text_list[0]);
-				stringptr = memcpy(malloc(len + 1), text_list[0], len + 1);
-				XFreeStringList(text_list);
-			}
-		}
-		else {
-			/* property is encoded in a format we don't understand */
-			fprintf(stderr,
-			        "%s: Encoding not STRING or COMPOUND_TEXT; unable to decode property %s of window ID %lx.\n",
-			        ProgramName, XGetAtomName(dpy, prop), w);
-			stringptr = NULL;
-		}
-		XFree(text_prop.value);
-	}
-	else {
-		stringptr = NULL;
-	}
-
-	return stringptr;
-}
-
-void FreeWMPropertyString(char *prop)
-{
-	if(prop && (char *)prop != NoName) {
-		free(prop);
-	}
-}
-
-static void ConstrainLeftTop(int *value, int border)
-{
-	if(*value < border) {
-		if(Scr->MoveOffResistance < 0 ||
-		                *value > border - Scr->MoveOffResistance) {
-			*value = border;
-		}
-		else if(Scr->MoveOffResistance > 0 &&
-		                *value <= border - Scr->MoveOffResistance) {
-			*value = *value + Scr->MoveOffResistance;
-		}
-	}
-}
-
-static void ConstrainRightBottom(int *value, int size1, int border, int size2)
-{
-	if(*value + size1 > size2 - border) {
-		if(Scr->MoveOffResistance < 0 ||
-		                *value + size1 < size2 - border + Scr->MoveOffResistance) {
-			*value = size2 - size1 - border;
-		}
-		else if(Scr->MoveOffResistance > 0 &&
-		                *value + size1 >= size2 - border + Scr->MoveOffResistance) {
-			*value = *value - Scr->MoveOffResistance;
-		}
-	}
-}
-
-void ConstrainByBorders1(int *left, int width, int *top, int height)
-{
-	ConstrainRightBottom(left, width, Scr->BorderRight, Scr->rootw);
-	ConstrainLeftTop(left, Scr->BorderLeft);
-	ConstrainRightBottom(top, height, Scr->BorderBottom, Scr->rooth);
-	ConstrainLeftTop(top, Scr->BorderTop);
-}
-
-void ConstrainByBorders(TwmWindow *twmwin,
-                        int *left, int width, int *top, int height)
-{
-	if(twmwin->winbox) {
-		XWindowAttributes attr;
-		XGetWindowAttributes(dpy, twmwin->winbox->window, &attr);
-		ConstrainRightBottom(left, width, 0, attr.width);
-		ConstrainLeftTop(left, 0);
-		ConstrainRightBottom(top, height, 0, attr.height);
-		ConstrainLeftTop(top, 0);
-	}
-	else {
-		ConstrainByBorders1(left, width, top, height);
-	}
-}
-
-
 /*
  * A safe strncpy(), which always ensures NUL-termination.
  *
@@ -1141,71 +654,3 @@
 	strncpy(dest, src, size - 1);
 	dest[size - 1] = '\0';
 }
-
-
-/*
- * Window mapped on some virtual screen?
- */
-bool
-visible(const TwmWindow *tmp_win)
-{
-	return (tmp_win->vs != NULL);
-}
-
-
-/*
- * Various code paths do a dance of "mask off notifications of event type
- * ; do something that triggers that event (but we're doing it, so we
- * don't need the notification) ; restore previous mask".  So have some
- * util funcs to make it more visually obvious.
- *
- * e.g.:
- *     long prev_mask = mask_out_event(w, PropertyChangeMask);
- *     do_something_that_changes_properties();
- *     restore_mask(prev_mask);
- *
- * We're cheating a little with the -1 return on mask_out_event(), as
- * that's theoretically valid for the data type.  It's not as far as I
- * can tell for X or us though; having all the bits set (well, I guess
- * I'm assuming 2s-complement too) is pretty absurd, and there are only
- * 25 defined bits in Xlib, so even on 32-bit systems, it shouldn't fill
- * up long.
- */
-long
-mask_out_event(Window w, long ignore_event)
-{
-	XWindowAttributes wattr;
-
-	/* Get current mask */
-	if(XGetWindowAttributes(dpy, w, &wattr) == 0) {
-		return -1;
-	}
-
-	/*
-	 * If we're ignoring nothing, nothing to do.  This is probably not
-	 * strictly speaking a useful thing to ask for in general, but it's
-	 * the right thing for us to do if we're asked to do nothing.
-	 */
-	if(ignore_event == 0) {
-		return wattr.your_event_mask;
-	}
-
-	/* Delegate */
-	return mask_out_event_mask(w, ignore_event, wattr.your_event_mask);
-}
-
-long
-mask_out_event_mask(Window w, long ignore_event, long curmask)
-{
-	/* Set to the current, minus what we're wanting to ignore */
-	XSelectInput(dpy, w, (curmask & ~ignore_event));
-
-	/* Return what it was */
-	return curmask;
-}
-
-int
-restore_mask(Window w, long restore)
-{
-	return XSelectInput(dpy, w, restore);
-}

=== modified file 'util.h'
--- util.h	2016-09-11 15:41:36 +0000
+++ util.h	2016-09-20 03:14:43 +0000
@@ -90,9 +90,6 @@
 #define Tolower(c)      tolower((int)(unsigned char)(c))
 #define Toupper(c)      toupper((int)(unsigned char)(c))
 
-void     MoveOutline(Window root,
-                     int x, int y, int width, int height,
-                     int bw, int th);
 char     *ExpandFilename(const char *name);
 char     *ExpandFilePath(char *path);
 
@@ -100,29 +97,15 @@
 void GetShadeColors(ColorPair *cp);
 void GetFont(MyFont *font);
 bool UpdateFont(MyFont *font, int height);
-void SetFocusVisualAttributes(TwmWindow *tmp_win, bool focus);
 #if 0
 void move_to_after(TwmWindow *t, TwmWindow *after);
 #endif
-void SetFocus(TwmWindow *tmp_win, Time tim);
 void AdoptWindow(void);
 void RescueWindows(void);
 void DebugTrace(char *file);
-void SetBorderCursor(TwmWindow *tmp_win, int x, int y);
-
-
-unsigned char *GetWMPropertyString(Window w, Atom prop);
-void FreeWMPropertyString(char *prop);
-void ConstrainByBorders1(int *left, int width, int *top, int height);
-void ConstrainByBorders(TwmWindow *twmwin,
-                        int *left, int width,
-                        int *top, int height);
+
+
 void safe_strncpy(char *dest, const char *src, size_t size);
-bool visible(const TwmWindow *tmp_win);
-
-long mask_out_event(Window w, long ignore_event);
-long mask_out_event_mask(Window w, long ignore_event, long curmask);
-int restore_mask(Window w, long restore);
 
 extern FILE *tracefile;
 

=== modified file 'vscreen.c'
--- vscreen.c	2016-08-13 07:22:55 +0000
+++ vscreen.c	2016-09-18 03:01:17 +0000
@@ -36,7 +36,7 @@
 #include "icons.h"
 #include "otp.h"
 #include "screen.h"
-#include "util.h"
+#include "win_utils.h"
 
 
 static void DisplayWinUnchecked(VirtualScreen *vs, TwmWindow *tmp_win);

=== modified file 'win_iconify.c'
--- win_iconify.c	2016-09-07 07:17:07 +0000
+++ win_iconify.c	2016-09-18 01:41:09 +0000
@@ -8,7 +8,6 @@
 
 #include <X11/extensions/shape.h>
 
-#include "add_window.h"
 #include "events.h"
 #include "functions.h"
 #include "iconmgr.h"
@@ -18,6 +17,8 @@
 #include "screen.h"
 #include "util.h"
 #include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 
 

=== added file 'win_ops.c'
--- win_ops.c	1970-01-01 00:00:00 +0000
+++ win_ops.c	2016-09-20 03:03:41 +0000
@@ -0,0 +1,407 @@
+/*
+ * Various operations done on windows.
+ */
+
+#include "ctwm.h"
+
+#include <stdio.h>
+
+#include "animate.h"
+#include "colormaps.h"
+#include "decorations.h"
+#include "drawing.h"
+#include "events.h"
+#include "iconmgr.h"
+#include "image.h"
+#include "otp.h"
+#include "screen.h"
+#include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
+
+
+/*
+ * Update the visuals of a window (e.g., its own decorations and its
+ * representation in the icon manager) for having/losing focus.
+ *
+ * Formerly in util.c
+ */
+void
+SetFocusVisualAttributes(TwmWindow *tmp_win, bool focus)
+{
+	if(! tmp_win) {
+		return;
+	}
+
+	if(focus == tmp_win->hasfocusvisible) {
+		return;
+	}
+	if(tmp_win->highlight) {
+		if(Scr->use3Dborders) {
+			PaintBorders(tmp_win, focus);
+		}
+		else {
+			if(focus) {
+				XSetWindowBorder(dpy, tmp_win->frame, tmp_win->borderC.back);
+				if(tmp_win->title_w) {
+					XSetWindowBorder(dpy, tmp_win->title_w, tmp_win->borderC.back);
+				}
+			}
+			else {
+				/*
+				 * XXX It seems possible this could be replaced by a
+				 * single global 'gray' pixmap; I don't think it actually
+				 * varies per window, and I don't see any obvious reason
+				 * it can't be reused, so we may be able to save an
+				 * allocation for each window by doing so...
+				 */
+				XSetWindowBorderPixmap(dpy, tmp_win->frame, tmp_win->gray);
+				if(tmp_win->title_w) {
+					XSetWindowBorderPixmap(dpy, tmp_win->title_w, tmp_win->gray);
+				}
+			}
+		}
+	}
+
+	if(focus) {
+		bool hil = false;
+
+		if(tmp_win->lolite_wl) {
+			XUnmapWindow(dpy, tmp_win->lolite_wl);
+		}
+		if(tmp_win->lolite_wr) {
+			XUnmapWindow(dpy, tmp_win->lolite_wr);
+		}
+		if(tmp_win->hilite_wl) {
+			XMapWindow(dpy, tmp_win->hilite_wl);
+			hil = true;
+		}
+		if(tmp_win->hilite_wr) {
+			XMapWindow(dpy, tmp_win->hilite_wr);
+			hil = true;
+		}
+		if(hil && tmp_win->HiliteImage && tmp_win->HiliteImage->next) {
+			MaybeAnimate = true;
+		}
+		if(tmp_win->iconmanagerlist) {
+			ActiveIconManager(tmp_win->iconmanagerlist);
+		}
+	}
+	else {
+		if(tmp_win->hilite_wl) {
+			XUnmapWindow(dpy, tmp_win->hilite_wl);
+		}
+		if(tmp_win->hilite_wr) {
+			XUnmapWindow(dpy, tmp_win->hilite_wr);
+		}
+		if(tmp_win->lolite_wl) {
+			XMapWindow(dpy, tmp_win->lolite_wl);
+		}
+		if(tmp_win->lolite_wr) {
+			XMapWindow(dpy, tmp_win->lolite_wr);
+		}
+		if(tmp_win->iconmanagerlist) {
+			NotActiveIconManager(tmp_win->iconmanagerlist);
+		}
+	}
+	if(Scr->use3Dtitles && Scr->SunkFocusWindowTitle && tmp_win->title_height) {
+		ButtonState bs;
+
+		bs = focus ? on : off;
+		Draw3DBorder(tmp_win->title_w, Scr->TBInfo.titlex, 0,
+		             tmp_win->title_width - Scr->TBInfo.titlex -
+		             Scr->TBInfo.rightoff - Scr->TitlePadding,
+		             Scr->TitleHeight, Scr->TitleShadowDepth,
+		             tmp_win->title, bs, false, false);
+	}
+	tmp_win->hasfocusvisible = focus;
+}
+
+
+/*
+ * Shift the focus to a given window, and do whatever subsidiary ops that
+ * entails.
+ *
+ * Formerly in util.c
+ */
+void
+SetFocus(TwmWindow *tmp_win, Time tim)
+{
+	Window w = (tmp_win ? tmp_win->w : PointerRoot);
+	bool f_iconmgr = false;
+
+	if(Scr->Focus && (Scr->Focus->isiconmgr)) {
+		f_iconmgr = true;
+	}
+	if(Scr->SloppyFocus && (w == PointerRoot) && (!f_iconmgr)) {
+		return;
+	}
+
+	XSetInputFocus(dpy, w, RevertToPointerRoot, tim);
+#ifdef EWMH
+	EwmhSet_NET_ACTIVE_WINDOW(w);
+#endif
+	if(Scr->Focus == tmp_win) {
+		return;
+	}
+
+	if(Scr->Focus) {
+		if(Scr->Focus->AutoSqueeze && !Scr->Focus->squeezed) {
+			AutoSqueeze(Scr->Focus);
+		}
+		SetFocusVisualAttributes(Scr->Focus, false);
+	}
+	if(tmp_win)    {
+		if(tmp_win->AutoSqueeze && tmp_win->squeezed) {
+			AutoSqueeze(tmp_win);
+		}
+		SetFocusVisualAttributes(tmp_win, true);
+	}
+	Scr->Focus = tmp_win;
+}
+
+
+/*
+ * Move the focus straight to the root, with associated cleanup.
+ *
+ * Formerly in menus.c
+ */
+void FocusOnRoot(void)
+{
+	SetFocus(NULL, EventTime);
+	InstallColormaps(0, &Scr->RootColormaps);
+	if(! Scr->ClickToFocus) {
+		Scr->FocusRoot = true;
+	}
+}
+
+
+/*
+ * Handle doing squeezing bits for AutoSqueeze{} windows.
+ *
+ * Formerly in menus.c
+ */
+void
+AutoSqueeze(TwmWindow *tmp_win)
+{
+	if(tmp_win->isiconmgr) {
+		return;
+	}
+	if(Scr->RaiseWhenAutoUnSqueeze && tmp_win->squeezed) {
+		OtpRaise(tmp_win, WinWin);
+	}
+	Squeeze(tmp_win);
+}
+
+
+/*
+ * Toggle a window's squeezed state.
+ *
+ * Formerly in menus.c
+ */
+void
+Squeeze(TwmWindow *tmp_win)
+{
+	long fx, fy, savex, savey;
+	int  neww, newh;
+	bool south;
+	int  grav = ((tmp_win->hints.flags & PWinGravity)
+	             ? tmp_win->hints.win_gravity : NorthWestGravity);
+	long eventMask;
+	if(tmp_win->squeezed) {
+		tmp_win->squeezed = False;
+#ifdef EWMH
+		EwmhSet_NET_WM_STATE(tmp_win, EWMH_STATE_SHADED);
+#endif /* EWMH */
+		if(!tmp_win->isicon) {
+			XMapWindow(dpy, tmp_win->w);
+		}
+		SetupWindow(tmp_win, tmp_win->actual_frame_x, tmp_win->actual_frame_y,
+		            tmp_win->actual_frame_width, tmp_win->actual_frame_height, -1);
+		ReMapTransients(tmp_win);
+		return;
+	}
+
+	newh = tmp_win->title_height + 2 * tmp_win->frame_bw3D;
+	if(newh < 3) {
+		XBell(dpy, 0);
+		return;
+	}
+	switch(grav) {
+		case SouthWestGravity :
+		case SouthGravity :
+		case SouthEastGravity :
+			south = true;
+			break;
+		default :
+			south = false;
+			break;
+	}
+	if(tmp_win->title_height && !tmp_win->AlwaysSqueezeToGravity) {
+		south = false;
+	}
+
+	tmp_win->squeezed = true;
+	tmp_win->actual_frame_width  = tmp_win->frame_width;
+	tmp_win->actual_frame_height = tmp_win->frame_height;
+	savex = fx = tmp_win->frame_x;
+	savey = fy = tmp_win->frame_y;
+	neww  = tmp_win->actual_frame_width;
+	if(south) {
+		fy += tmp_win->frame_height - newh;
+	}
+	if(tmp_win->squeeze_info) {
+		fx  += tmp_win->title_x - tmp_win->frame_bw3D;
+		neww = tmp_win->title_width + 2 * (tmp_win->frame_bw + tmp_win->frame_bw3D);
+	}
+
+	eventMask = mask_out_event(tmp_win->w, StructureNotifyMask);
+#ifdef EWMH
+	EwmhSet_NET_WM_STATE(tmp_win, EWMH_STATE_SHADED);
+#endif /* EWMH */
+	XUnmapWindow(dpy, tmp_win->w);
+	restore_mask(tmp_win->w, eventMask);
+
+	if(fx + neww >= Scr->rootw - Scr->BorderRight) {
+		fx = Scr->rootw - Scr->BorderRight - neww;
+	}
+	if(fy + newh >= Scr->rooth - Scr->BorderBottom) {
+		fy = Scr->rooth - Scr->BorderBottom - newh;
+	}
+	SetupWindow(tmp_win, fx, fy, neww, newh, -1);
+	tmp_win->actual_frame_x = savex;
+	tmp_win->actual_frame_y = savey;
+
+	/* Now make the group members disappear */
+	UnmapTransients(tmp_win, false, eventMask);
+}
+
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *      MoveOutline - move a window outline
+ *
+ *  Inputs:
+ *      root        - the window we are outlining
+ *      x           - upper left x coordinate
+ *      y           - upper left y coordinate
+ *      width       - the width of the rectangle
+ *      height      - the height of the rectangle
+ *      bw          - the border width of the frame
+ *      th          - title height
+ *
+ ***********************************************************************
+ */
+void
+MoveOutline(Window root, int x, int y, int width, int height, int bw, int th)
+{
+	static int  lastx = 0;
+	static int  lasty = 0;
+	static int  lastWidth = 0;
+	static int  lastHeight = 0;
+	static int  lastBW = 0;
+	static int  lastTH = 0;
+	int         xl, xr, yt, yb, xinnerl, xinnerr, yinnert, yinnerb;
+	int         xthird, ythird;
+	XSegment    outline[18];
+	XSegment   *r;
+
+	if(x == lastx && y == lasty && width == lastWidth && height == lastHeight
+	                && lastBW == bw && th == lastTH) {
+		return;
+	}
+
+	r = outline;
+
+#define DRAWIT() \
+    if (lastWidth || lastHeight)                        \
+    {                                                   \
+        xl = lastx;                                     \
+        xr = lastx + lastWidth - 1;                     \
+        yt = lasty;                                     \
+        yb = lasty + lastHeight - 1;                    \
+        xinnerl = xl + lastBW;                          \
+        xinnerr = xr - lastBW;                          \
+        yinnert = yt + lastTH + lastBW;                 \
+        yinnerb = yb - lastBW;                          \
+        xthird = (xinnerr - xinnerl) / 3;               \
+        ythird = (yinnerb - yinnert) / 3;               \
+                                                        \
+        r->x1 = xl;                                     \
+        r->y1 = yt;                                     \
+        r->x2 = xr;                                     \
+        r->y2 = yt;                                     \
+        r++;                                            \
+                                                        \
+        r->x1 = xl;                                     \
+        r->y1 = yb;                                     \
+        r->x2 = xr;                                     \
+        r->y2 = yb;                                     \
+        r++;                                            \
+                                                        \
+        r->x1 = xl;                                     \
+        r->y1 = yt;                                     \
+        r->x2 = xl;                                     \
+        r->y2 = yb;                                     \
+        r++;                                            \
+                                                        \
+        r->x1 = xr;                                     \
+        r->y1 = yt;                                     \
+        r->x2 = xr;                                     \
+        r->y2 = yb;                                     \
+        r++;                                            \
+                                                        \
+        r->x1 = xinnerl + xthird;                       \
+        r->y1 = yinnert;                                \
+        r->x2 = r->x1;                                  \
+        r->y2 = yinnerb;                                \
+        r++;                                            \
+                                                        \
+        r->x1 = xinnerl + (2 * xthird);                 \
+        r->y1 = yinnert;                                \
+        r->x2 = r->x1;                                  \
+        r->y2 = yinnerb;                                \
+        r++;                                            \
+                                                        \
+        r->x1 = xinnerl;                                \
+        r->y1 = yinnert + ythird;                       \
+        r->x2 = xinnerr;                                \
+        r->y2 = r->y1;                                  \
+        r++;                                            \
+                                                        \
+        r->x1 = xinnerl;                                \
+        r->y1 = yinnert + (2 * ythird);                 \
+        r->x2 = xinnerr;                                \
+        r->y2 = r->y1;                                  \
+        r++;                                            \
+                                                        \
+        if (lastTH != 0) {                              \
+            r->x1 = xl;                                 \
+            r->y1 = yt + lastTH;                        \
+            r->x2 = xr;                                 \
+            r->y2 = r->y1;                              \
+            r++;                                        \
+        }                                               \
+    }
+
+	/* undraw the old one, if any */
+	DRAWIT();
+
+	lastx = x;
+	lasty = y;
+	lastWidth = width;
+	lastHeight = height;
+	lastBW = bw;
+	lastTH = th;
+
+	/* draw the new one, if any */
+	DRAWIT();
+
+#undef DRAWIT
+
+
+	if(r != outline) {
+		XDrawSegments(dpy, root, Scr->DrawGC, outline, r - outline);
+	}
+}

=== added file 'win_ops.h'
--- win_ops.h	1970-01-01 00:00:00 +0000
+++ win_ops.h	2016-09-20 03:03:41 +0000
@@ -0,0 +1,17 @@
+/*
+ * Window operations
+ */
+#ifndef _CTWM_WIN_OPS_H
+#define _CTWM_WIN_OPS_H
+
+
+void SetFocusVisualAttributes(TwmWindow *tmp_win, bool focus);
+void SetFocus(TwmWindow *tmp_win, Time tim);
+void FocusOnRoot(void);
+void AutoSqueeze(TwmWindow *tmp_win);
+void Squeeze(TwmWindow *tmp_win);
+void MoveOutline(Window root, int x, int y, int width, int height,
+                 int bw, int th);
+
+
+#endif /* _CTWM_WIN_OPS_H */

=== added file 'win_utils.c'
--- win_utils.c	1970-01-01 00:00:00 +0000
+++ win_utils.c	2016-09-20 03:14:43 +0000
@@ -0,0 +1,729 @@
+/*
+ * Window-handling utility funcs
+ */
+
+#include "ctwm.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+
+#include <X11/Xatom.h>
+
+#include "add_window.h" // NoName
+#include "ctwm_atoms.h"
+#include "decorations.h"
+#include "drawing.h"
+#include "icons.h"
+#include "screen.h"
+#include "util.h"
+#include "win_utils.h"
+
+
+/*
+ * Fill in size hints for a window from WM_NORMAL_HINTS prop.
+ *
+ * Formerly in add_window.c
+ */
+void
+GetWindowSizeHints(TwmWindow *tmp)
+{
+	long supplied = 0;
+	XSizeHints *hints = &tmp->hints;
+
+	if(!XGetWMNormalHints(dpy, tmp->w, hints, &supplied)) {
+		hints->flags = 0;
+	}
+
+	if(hints->flags & PResizeInc) {
+		if(hints->width_inc == 0) {
+			hints->width_inc = 1;
+		}
+		if(hints->height_inc == 0) {
+			hints->height_inc = 1;
+		}
+	}
+
+	if(!(supplied & PWinGravity) && (hints->flags & USPosition)) {
+		static int gravs[] = { SouthEastGravity, SouthWestGravity,
+		                       NorthEastGravity, NorthWestGravity
+		                     };
+		int right =  tmp->attr.x + tmp->attr.width + 2 * tmp->old_bw;
+		int bottom = tmp->attr.y + tmp->attr.height + 2 * tmp->old_bw;
+		hints->win_gravity =
+		        gravs[((Scr->rooth - bottom <
+		                tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 2) |
+		              ((Scr->rootw - right   <
+		                tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 1)];
+		hints->flags |= PWinGravity;
+	}
+
+	/* Check for min size < max size */
+	if((hints->flags & (PMinSize | PMaxSize)) == (PMinSize | PMaxSize)) {
+		if(hints->max_width < hints->min_width) {
+			if(hints->max_width > 0) {
+				hints->min_width = hints->max_width;
+			}
+			else if(hints->min_width > 0) {
+				hints->max_width = hints->min_width;
+			}
+			else {
+				hints->max_width = hints->min_width = 1;
+			}
+		}
+
+		if(hints->max_height < hints->min_height) {
+			if(hints->max_height > 0) {
+				hints->min_height = hints->max_height;
+			}
+			else if(hints->min_height > 0) {
+				hints->max_height = hints->min_height;
+			}
+			else {
+				hints->max_height = hints->min_height = 1;
+			}
+		}
+	}
+}
+
+
+/*
+ * Fill in info from WM_PROTOCOLS property
+ *
+ * Formerly in add_window.c
+ */
+void
+FetchWmProtocols(TwmWindow *tmp)
+{
+	unsigned long flags = 0L;
+	Atom *protocols = NULL;
+	int n;
+
+	if(XGetWMProtocols(dpy, tmp->w, &protocols, &n)) {
+		int i;
+		Atom *ap;
+
+		for(i = 0, ap = protocols; i < n; i++, ap++) {
+			if(*ap == XA_WM_TAKE_FOCUS) {
+				flags |= DoesWmTakeFocus;
+			}
+			if(*ap == XA_WM_SAVE_YOURSELF) {
+				flags |= DoesWmSaveYourself;
+			}
+			if(*ap == XA_WM_DELETE_WINDOW) {
+				flags |= DoesWmDeleteWindow;
+			}
+		}
+		if(protocols) {
+			XFree(protocols);
+		}
+	}
+	tmp->protocols = flags;
+}
+
+
+/*
+ * Figure signs for calculating location offsets for a window dependent
+ * on its gravity.
+ *
+ * Depending on how its gravity is set, offsets to window coordinates for
+ * e.g. border widths may need to move either down (right) or up (left).
+ * Or possibly not at all.  So we write multipliers into passed vars for
+ * callers.
+ *
+ * Formerly in add_window.c
+ */
+void
+GetGravityOffsets(TwmWindow *tmp, int *xp, int *yp)
+{
+	static struct _gravity_offset {
+		int x, y;
+	} gravity_offsets[] = {
+		[ForgetGravity]    = {  0,  0 },
+		[NorthWestGravity] = { -1, -1 },
+		[NorthGravity]     = {  0, -1 },
+		[NorthEastGravity] = {  1, -1 },
+		[WestGravity]      = { -1,  0 },
+		[CenterGravity]    = {  0,  0 },
+		[EastGravity]      = {  1,  0 },
+		[SouthWestGravity] = { -1,  1 },
+		[SouthGravity]     = {  0,  1 },
+		[SouthEastGravity] = {  1,  1 },
+		[StaticGravity]    = {  0,  0 },
+	};
+	int g = ((tmp->hints.flags & PWinGravity)
+	         ? tmp->hints.win_gravity : NorthWestGravity);
+
+	if(g < ForgetGravity || g > StaticGravity) {
+		*xp = *yp = 0;
+	}
+	else {
+		*xp = gravity_offsets[g].x;
+		*yp = gravity_offsets[g].y;
+	}
+}
+
+
+/*
+ * Finds the TwmWindow structure associated with a Window (if any), or
+ * NULL.
+ *
+ * This is a relatively cheap function since it does not involve
+ * communication with the server. Probably faster than walking the list
+ * of TwmWindows, since the lookup is by a hash table.
+ *
+ * Formerly in add_window.c
+ */
+TwmWindow *
+GetTwmWindow(Window w)
+{
+	TwmWindow *twmwin;
+	int stat;
+
+	stat = XFindContext(dpy, w, TwmContext, (XPointer *)&twmwin);
+	if(stat == XCNOENT) {
+		twmwin = NULL;
+	}
+
+	return twmwin;
+}
+
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *      GetWMPropertyString - Get Window Manager text property and
+ *                              convert it to a string.
+ *
+ *  Returned Value:
+ *      (char *) - pointer to the malloc'd string or NULL
+ *
+ *  Inputs:
+ *      w       - the id of the window whose property is to be retrieved
+ *      prop    - property atom (typically WM_NAME or WM_ICON_NAME)
+ *
+ ***********************************************************************
+ *
+ * Formerly in util.c
+ */
+unsigned char *
+GetWMPropertyString(Window w, Atom prop)
+{
+	XTextProperty       text_prop;
+	char                **text_list;
+	int                 text_list_count;
+	unsigned char       *stringptr;
+	int                 status, len = -1;
+
+	(void)XGetTextProperty(dpy, w, &text_prop, prop);
+	if(text_prop.value != NULL) {
+		if(text_prop.encoding == XA_STRING
+		                || text_prop.encoding == XA_COMPOUND_TEXT) {
+			/* property is encoded as compound text - convert to locale string */
+			status = XmbTextPropertyToTextList(dpy, &text_prop,
+			                                   &text_list, &text_list_count);
+			if(text_list_count == 0) {
+				stringptr = NULL;
+			}
+			else if(text_list == NULL) {
+				stringptr = NULL;
+			}
+			else if(text_list [0] == NULL) {
+				stringptr = NULL;
+			}
+			else if(status < 0 || text_list_count < 0) {
+				switch(status) {
+					case XConverterNotFound:
+						fprintf(stderr,
+						        "%s: Converter not found; unable to convert property %s of window ID %lx.\n",
+						        ProgramName, XGetAtomName(dpy, prop), w);
+						break;
+					case XNoMemory:
+						fprintf(stderr,
+						        "%s: Insufficient memory; unable to convert property %s of window ID %lx.\n",
+						        ProgramName, XGetAtomName(dpy, prop), w);
+						break;
+					case XLocaleNotSupported:
+						fprintf(stderr,
+						        "%s: Locale not supported; unable to convert property %s of window ID %lx.\n",
+						        ProgramName, XGetAtomName(dpy, prop), w);
+						break;
+				}
+				stringptr = NULL;
+				/*
+				   don't call XFreeStringList - text_list appears to have
+				   invalid address if status is bad
+				   XFreeStringList(text_list);
+				*/
+			}
+			else {
+				len = strlen(text_list[0]);
+				stringptr = memcpy(malloc(len + 1), text_list[0], len + 1);
+				XFreeStringList(text_list);
+			}
+		}
+		else {
+			/* property is encoded in a format we don't understand */
+			fprintf(stderr,
+			        "%s: Encoding not STRING or COMPOUND_TEXT; unable to decode property %s of window ID %lx.\n",
+			        ProgramName, XGetAtomName(dpy, prop), w);
+			stringptr = NULL;
+		}
+		XFree(text_prop.value);
+	}
+	else {
+		stringptr = NULL;
+	}
+
+	return stringptr;
+}
+
+
+/*
+ * Cleanup something stored that we got from the above originally.
+ *
+ * Formerly in util.c
+ */
+void
+FreeWMPropertyString(char *prop)
+{
+	if(prop && (char *)prop != NoName) {
+		free(prop);
+	}
+}
+
+
+/*
+ * Window mapped on some virtual screen?
+ *
+ * Formerly in util.c
+ */
+bool
+visible(const TwmWindow *tmp_win)
+{
+	return (tmp_win->vs != NULL);
+}
+
+
+/*
+ * Various code paths do a dance of "mask off notifications of event type
+ * ; do something that triggers that event (but we're doing it, so we
+ * don't need the notification) ; restore previous mask".  So have some
+ * util funcs to make it more visually obvious.
+ *
+ * e.g.:
+ *     long prev_mask = mask_out_event(w, PropertyChangeMask);
+ *     do_something_that_changes_properties();
+ *     restore_mask(prev_mask);
+ *
+ * We're cheating a little with the -1 return on mask_out_event(), as
+ * that's theoretically valid for the data type.  It's not as far as I
+ * can tell for X or us though; having all the bits set (well, I guess
+ * I'm assuming 2s-complement too) is pretty absurd, and there are only
+ * 25 defined bits in Xlib, so even on 32-bit systems, it shouldn't fill
+ * up long.
+ */
+long
+mask_out_event(Window w, long ignore_event)
+{
+	XWindowAttributes wattr;
+
+	/* Get current mask */
+	if(XGetWindowAttributes(dpy, w, &wattr) == 0) {
+		return -1;
+	}
+
+	/*
+	 * If we're ignoring nothing, nothing to do.  This is probably not
+	 * strictly speaking a useful thing to ask for in general, but it's
+	 * the right thing for us to do if we're asked to do nothing.
+	 */
+	if(ignore_event == 0) {
+		return wattr.your_event_mask;
+	}
+
+	/* Delegate */
+	return mask_out_event_mask(w, ignore_event, wattr.your_event_mask);
+}
+
+long
+mask_out_event_mask(Window w, long ignore_event, long curmask)
+{
+	/* Set to the current, minus what we're wanting to ignore */
+	XSelectInput(dpy, w, (curmask & ~ignore_event));
+
+	/* Return what it was */
+	return curmask;
+}
+
+int
+restore_mask(Window w, long restore)
+{
+	return XSelectInput(dpy, w, restore);
+}
+
+
+/*
+ * Setting and getting WM_STATE property.
+ *
+ * x-ref ICCCM section 4.1.3.1
+ * https://tronche.com/gui/x/icccm/sec-4.html#s-4.1.3.1
+ *
+ * XXX These should probably be named more alike, as they're
+ * complementary ops.
+ */
+void
+SetMapStateProp(TwmWindow *tmp_win, int state)
+{
+	unsigned long data[2];              /* "suggested" by ICCCM version 1 */
+
+	data[0] = (unsigned long) state;
+	data[1] = (unsigned long)(tmp_win->iconify_by_unmapping ? None :
+	                          (tmp_win->icon ? tmp_win->icon->w : None));
+
+	XChangeProperty(dpy, tmp_win->w, XA_WM_STATE, XA_WM_STATE, 32,
+	                PropModeReplace, (unsigned char *) data, 2);
+}
+
+
+bool
+GetWMState(Window w, int *statep, Window *iwp)
+{
+	Atom actual_type;
+	int actual_format;
+	unsigned long nitems, bytesafter;
+	unsigned long *datap = NULL;
+	bool retval = false;
+
+	if(XGetWindowProperty(dpy, w, XA_WM_STATE, 0L, 2L, False, XA_WM_STATE,
+	                      &actual_type, &actual_format, &nitems, &bytesafter,
+	                      (unsigned char **) &datap) != Success || !datap) {
+		return false;
+	}
+
+	if(nitems <= 2) {                   /* "suggested" by ICCCM version 1 */
+		*statep = (int) datap[0];
+		*iwp = (Window) datap[1];
+		retval = true;
+	}
+
+	XFree(datap);
+	return retval;
+}
+
+
+/*
+ * Display a window's position in the dimensions window.  This is used
+ * during various window positioning (during new window popups, moves,
+ * etc).
+ *
+ * This reuses the same window for the position as is used during
+ * resizing for the dimesions of the window in DisplaySize().  The
+ * innards of the funcs can probably be collapsed together a little, and
+ * the higher-level knowledge of Scr->SizeWindow (e.g., for unmapping
+ * after ths op is done) should probably be encapsulated a bit better.
+ */
+void
+DisplayPosition(const TwmWindow *_unused_tmp_win, int x, int y)
+{
+	char str [100];
+	char signx = '+';
+	char signy = '+';
+
+	if(x < 0) {
+		x = -x;
+		signx = '-';
+	}
+	if(y < 0) {
+		y = -y;
+		signy = '-';
+	}
+	sprintf(str, " %c%-4d %c%-4d ", signx, x, signy, y);
+	XRaiseWindow(dpy, Scr->SizeWindow);
+
+	Draw3DBorder(Scr->SizeWindow, 0, 0,
+	             Scr->SizeStringOffset + Scr->SizeStringWidth + SIZE_HINDENT,
+	             Scr->SizeFont.height + SIZE_VINDENT * 2,
+	             2, Scr->DefaultC, off, false, false);
+
+	FB(Scr->DefaultC.fore, Scr->DefaultC.back);
+	XmbDrawImageString(dpy, Scr->SizeWindow, Scr->SizeFont.font_set,
+	                   Scr->NormalGC, Scr->SizeStringOffset,
+	                   Scr->SizeFont.ascent + SIZE_VINDENT , str, 13);
+}
+
+
+/*
+ * Various funcs for adjusting coordinates for windows based on
+ * resistances etc.
+ *
+ * XXX In desperate need of better commenting.
+ */
+void
+TryToPack(TwmWindow *tmp_win, int *x, int *y)
+{
+	TwmWindow   *t;
+	int         newx, newy;
+	int         w, h;
+	int         winw = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
+	int         winh = tmp_win->frame_height + 2 * tmp_win->frame_bw;
+
+	newx = *x;
+	newy = *y;
+	for(t = Scr->FirstWindow; t != NULL; t = t->next) {
+		if(t == tmp_win) {
+			continue;
+		}
+		if(t->winbox != tmp_win->winbox) {
+			continue;
+		}
+		if(t->vs != tmp_win->vs) {
+			continue;
+		}
+		if(!t->mapped) {
+			continue;
+		}
+
+		w = t->frame_width  + 2 * t->frame_bw;
+		h = t->frame_height + 2 * t->frame_bw;
+		if(newx >= t->frame_x + w) {
+			continue;
+		}
+		if(newy >= t->frame_y + h) {
+			continue;
+		}
+		if(newx + winw <= t->frame_x) {
+			continue;
+		}
+		if(newy + winh <= t->frame_y) {
+			continue;
+		}
+
+		if(newx + Scr->MovePackResistance > t->frame_x + w) {  /* left */
+			newx = MAX(newx, t->frame_x + w);
+			continue;
+		}
+		if(newx + winw < t->frame_x + Scr->MovePackResistance) {  /* right */
+			newx = MIN(newx, t->frame_x - winw);
+			continue;
+		}
+		if(newy + Scr->MovePackResistance > t->frame_y + h) {  /* top */
+			newy = MAX(newy, t->frame_y + h);
+			continue;
+		}
+		if(newy + winh < t->frame_y + Scr->MovePackResistance) {  /* bottom */
+			newy = MIN(newy, t->frame_y - winh);
+			continue;
+		}
+	}
+	*x = newx;
+	*y = newy;
+}
+
+
+/*
+ * Directionals for TryToPush_be().  These differ from the specs for
+ * jump/pack/fill in functions. because there's an indeterminate option.
+ */
+typedef enum {
+	PD_ANY,
+	PD_BOTTOM,
+	PD_LEFT,
+	PD_RIGHT,
+	PD_TOP,
+} PushDirection;
+static void TryToPush_be(TwmWindow *tmp_win, int x, int y, PushDirection dir);
+
+void
+TryToPush(TwmWindow *tmp_win, int x, int y)
+{
+	TryToPush_be(tmp_win, x, y, PD_ANY);
+}
+
+static void
+TryToPush_be(TwmWindow *tmp_win, int x, int y, PushDirection dir)
+{
+	TwmWindow   *t;
+	int         newx, newy, ndir;
+	bool        move;
+	int         w, h;
+	int         winw = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
+	int         winh = tmp_win->frame_height + 2 * tmp_win->frame_bw;
+
+	for(t = Scr->FirstWindow; t != NULL; t = t->next) {
+		if(t == tmp_win) {
+			continue;
+		}
+		if(t->winbox != tmp_win->winbox) {
+			continue;
+		}
+		if(t->vs != tmp_win->vs) {
+			continue;
+		}
+		if(!t->mapped) {
+			continue;
+		}
+
+		w = t->frame_width  + 2 * t->frame_bw;
+		h = t->frame_height + 2 * t->frame_bw;
+		if(x >= t->frame_x + w) {
+			continue;
+		}
+		if(y >= t->frame_y + h) {
+			continue;
+		}
+		if(x + winw <= t->frame_x) {
+			continue;
+		}
+		if(y + winh <= t->frame_y) {
+			continue;
+		}
+
+		move = false;
+		if((dir == PD_ANY || dir == PD_LEFT) &&
+		                (x + Scr->MovePackResistance > t->frame_x + w)) {
+			newx = x - w;
+			newy = t->frame_y;
+			ndir = PD_LEFT;
+			move = true;
+		}
+		else if((dir == PD_ANY || dir == PD_RIGHT) &&
+		                (x + winw < t->frame_x + Scr->MovePackResistance)) {
+			newx = x + winw;
+			newy = t->frame_y;
+			ndir = PD_RIGHT;
+			move = true;
+		}
+		else if((dir == PD_ANY || dir == PD_TOP) &&
+		                (y + Scr->MovePackResistance > t->frame_y + h)) {
+			newx = t->frame_x;
+			newy = y - h;
+			ndir = PD_TOP;
+			move = true;
+		}
+		else if((dir == PD_ANY || dir == PD_BOTTOM) &&
+		                (y + winh < t->frame_y + Scr->MovePackResistance)) {
+			newx = t->frame_x;
+			newy = y + winh;
+			ndir = PD_BOTTOM;
+			move = true;
+		}
+		if(move) {
+			TryToPush_be(t, newx, newy, ndir);
+			TryToPack(t, &newx, &newy);
+			ConstrainByBorders(tmp_win,
+			                   &newx, t->frame_width  + 2 * t->frame_bw,
+			                   &newy, t->frame_height + 2 * t->frame_bw);
+			SetupWindow(t, newx, newy, t->frame_width, t->frame_height, -1);
+		}
+	}
+}
+
+
+void
+TryToGrid(TwmWindow *tmp_win, int *x, int *y)
+{
+	int w    = tmp_win->frame_width  + 2 * tmp_win->frame_bw;
+	int h    = tmp_win->frame_height + 2 * tmp_win->frame_bw;
+	int grav = ((tmp_win->hints.flags & PWinGravity)
+	            ? tmp_win->hints.win_gravity : NorthWestGravity);
+
+	switch(grav) {
+		case ForgetGravity :
+		case StaticGravity :
+		case NorthWestGravity :
+		case NorthGravity :
+		case WestGravity :
+		case CenterGravity :
+			*x = ((*x - Scr->BorderLeft) / Scr->XMoveGrid) * Scr->XMoveGrid
+			     + Scr->BorderLeft;
+			*y = ((*y - Scr->BorderTop) / Scr->YMoveGrid) * Scr->YMoveGrid
+			     + Scr->BorderTop;
+			break;
+		case NorthEastGravity :
+		case EastGravity :
+			*x = (((*x + w - Scr->BorderLeft) / Scr->XMoveGrid) *
+			      Scr->XMoveGrid) - w + Scr->BorderLeft;
+			*y = ((*y - Scr->BorderTop) / Scr->YMoveGrid) *
+			     Scr->YMoveGrid + Scr->BorderTop;
+			break;
+		case SouthWestGravity :
+		case SouthGravity :
+			*x = ((*x - Scr->BorderLeft) / Scr->XMoveGrid) * Scr->XMoveGrid
+			     + Scr->BorderLeft;
+			*y = (((*y + h - Scr->BorderTop) / Scr->YMoveGrid) * Scr->YMoveGrid)
+			     - h + Scr->BorderTop;
+			break;
+		case SouthEastGravity :
+			*x = (((*x + w - Scr->BorderLeft) / Scr->XMoveGrid) *
+			      Scr->XMoveGrid) - w + Scr->BorderLeft;
+			*y = (((*y + h - Scr->BorderTop) / Scr->YMoveGrid) *
+			      Scr->YMoveGrid) - h + Scr->BorderTop;
+			break;
+	}
+}
+
+
+
+/*
+ * Functions related to keeping windows from being placed off-screen (or
+ * off-screen too far).  Involved in handling of params like DontMoveOff
+ * and MoveOffResistance, etc.
+ */
+static void ConstrainLeftTop(int *value, int border);
+static void ConstrainRightBottom(int *value, int size1, int border, int size2);
+
+void
+ConstrainByBorders1(int *left, int width, int *top, int height)
+{
+	ConstrainRightBottom(left, width, Scr->BorderRight, Scr->rootw);
+	ConstrainLeftTop(left, Scr->BorderLeft);
+	ConstrainRightBottom(top, height, Scr->BorderBottom, Scr->rooth);
+	ConstrainLeftTop(top, Scr->BorderTop);
+}
+
+void
+ConstrainByBorders(TwmWindow *twmwin, int *left, int width,
+                   int *top, int height)
+{
+	if(twmwin->winbox) {
+		XWindowAttributes attr;
+		XGetWindowAttributes(dpy, twmwin->winbox->window, &attr);
+		ConstrainRightBottom(left, width, 0, attr.width);
+		ConstrainLeftTop(left, 0);
+		ConstrainRightBottom(top, height, 0, attr.height);
+		ConstrainLeftTop(top, 0);
+	}
+	else {
+		ConstrainByBorders1(left, width, top, height);
+	}
+}
+
+static void
+ConstrainLeftTop(int *value, int border)
+{
+	if(*value < border) {
+		if(Scr->MoveOffResistance < 0 ||
+		                *value > border - Scr->MoveOffResistance) {
+			*value = border;
+		}
+		else if(Scr->MoveOffResistance > 0 &&
+		                *value <= border - Scr->MoveOffResistance) {
+			*value = *value + Scr->MoveOffResistance;
+		}
+	}
+}
+
+static void
+ConstrainRightBottom(int *value, int size1, int border, int size2)
+{
+	if(*value + size1 > size2 - border) {
+		if(Scr->MoveOffResistance < 0 ||
+		                *value + size1 < size2 - border + Scr->MoveOffResistance) {
+			*value = size2 - size1 - border;
+		}
+		else if(Scr->MoveOffResistance > 0 &&
+		                *value + size1 >= size2 - border + Scr->MoveOffResistance) {
+			*value = *value - Scr->MoveOffResistance;
+		}
+	}
+}

=== added file 'win_utils.h'
--- win_utils.h	1970-01-01 00:00:00 +0000
+++ win_utils.h	2016-09-20 03:14:43 +0000
@@ -0,0 +1,29 @@
+/*
+ * Window util funcs
+ */
+#ifndef _CTWM_WIN_UTILS_H
+#define _CTWM_WIN_UTILS_H
+
+
+void GetWindowSizeHints(TwmWindow *tmp_win);
+void FetchWmProtocols(TwmWindow *tmp);
+void GetGravityOffsets(TwmWindow *tmp, int *xp, int *yp);
+TwmWindow *GetTwmWindow(Window w);
+unsigned char *GetWMPropertyString(Window w, Atom prop);
+void FreeWMPropertyString(char *prop);
+bool visible(const TwmWindow *tmp_win);
+long mask_out_event(Window w, long ignore_event);
+long mask_out_event_mask(Window w, long ignore_event, long curmask);
+int restore_mask(Window w, long restore);
+void SetMapStateProp(TwmWindow *tmp_win, int state);
+bool GetWMState(Window w, int *statep, Window *iwp);
+void DisplayPosition(const TwmWindow *_unused_tmp_win, int x, int y);
+void TryToPack(TwmWindow *tmp_win, int *x, int *y);
+void TryToPush(TwmWindow *tmp_win, int x, int y);
+void TryToGrid(TwmWindow *tmp_win, int *x, int *y);
+void ConstrainByBorders1(int *left, int width, int *top, int height);
+void ConstrainByBorders(TwmWindow *twmwin, int *left, int width,
+                        int *top, int height);
+
+
+#endif /* _CTWM_WIN_UTILS_H */

=== modified file 'windowbox.c'
--- windowbox.c	2016-07-31 11:37:38 +0000
+++ windowbox.c	2016-09-18 02:58:07 +0000
@@ -37,7 +37,7 @@
 #include "decorations.h"
 #include "resize.h"
 #include "windowbox.h"
-#include "util.h"
+#include "win_utils.h"
 
 name_list **addWindowBox(char *boxname, char *geometry)
 {

=== modified file 'workspace_manager.c'
--- workspace_manager.c	2016-09-03 04:29:20 +0000
+++ workspace_manager.c	2016-09-20 03:03:41 +0000
@@ -44,6 +44,8 @@
 #include "image.h"
 #include "drawing.h"
 #include "win_iconify.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
 

=== modified file 'workspace_utils.c'
--- workspace_utils.c	2016-08-13 09:29:32 +0000
+++ workspace_utils.c	2016-09-18 03:01:17 +0000
@@ -19,7 +19,8 @@
 #include "image.h"
 #include "otp.h"
 #include "screen.h"
-#include "util.h"
+#include "win_ops.h"
+#include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
 

